@@ -5,23 +5,36 @@ In this module there're implementations & tests of `SimpleHTTP`.
 use std::collections::VecDeque;
 use std::error::Error as StdError;
 use std::future::Future;
+use std::io::{self, Write};
 use std::pin::Pin;
 use std::result::Result as StdResult;
 use std::str::FromStr;
-use std::sync::{Arc, Mutex};
+use std::sync::{
+    atomic::{AtomicBool, Ordering},
+    Arc, Mutex,
+};
+use std::task::{Context, Poll, Waker};
+use std::thread;
 
 use http::method::Method;
 // use futures::TryStreamExt;
 // use hyper::body::HttpBody;
 use bytes::Bytes;
+// use futures::executor::block_on;
+use futures::executor::ThreadPool;
+use futures::prelude::*;
+use futures::Stream;
+// use futures::task::SpawnExt;
 use hyper::body::HttpBody;
 use hyper::client::{connect::Connect, HttpConnector};
 use hyper::header::{HeaderValue, CONTENT_TYPE};
-use hyper::{Body, Client, Error, HeaderMap, Request, Response, Result, Uri};
+use hyper::{Body, Client, HeaderMap, Request, Response, Result, Uri};
 use url::Url;
 
-use super::common::{PathParam, QueryParam};
-use super::simple_api::{BaseAPI, BaseService, BodySerializer, SimpleAPI};
+use super::common::{make_stream, PathParam, QueryParam, WriteForStream};
+use super::simple_api::{
+    APIMultipart, BaseAPI, BaseService, BodyDeserializer, BodySerializer, SimpleAPI,
+};
 use super::simple_http::{
     BaseClient, FormDataParseError, SimpleHTTP, SimpleHTTPResponse, DEFAULT_TIMEOUT_MILLISECOND,
 };
@@ -42,7 +55,189 @@ use multer;
 #[cfg(feature = "multipart")]
 use multer::Multipart;
 
-pub struct HyperClient<C, B>(Client<C, B>);
+#[derive(Clone)]
+pub struct WriteForBody {
+    // pub Box<Sender>
+    pub cached: Arc<Mutex<VecDeque<Bytes>>>,
+    pub waker: Arc<Mutex<Option<Waker>>>,
+    pub alive: Arc<Mutex<AtomicBool>>,
+}
+
+impl WriteForBody {
+    pub fn close(&self) {
+        self.alive.lock().unwrap().store(false, Ordering::SeqCst);
+
+        {
+            if let Some(waker) = self.waker.lock().unwrap().take() {
+                waker.wake()
+            }
+        }
+    }
+}
+
+impl Stream for WriteForBody {
+    type Item = StdResult<Bytes, Box<dyn StdError + Send + Sync>>;
+
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+        {
+            {
+                let mut cached = self.cached.lock().unwrap();
+                if !cached.is_empty() {
+                    self.waker.lock().unwrap().replace(cx.waker().clone());
+
+                    let d = cached.pop_front();
+                    println!("WriteForBody stream read content: {:?}", d.clone());
+                    return Poll::Ready(Some(Ok(d.unwrap())));
+                }
+            }
+            {
+                if !self.alive.lock().unwrap().load(Ordering::SeqCst) {
+                    println!("WriteForBody stream end");
+                    return Poll::Ready(None);
+                }
+            }
+        }
+
+        {
+            self.waker.lock().unwrap().replace(cx.waker().clone());
+            println!("WriteForBody stream pending");
+            Poll::Pending
+        }
+    }
+}
+
+impl io::Write for WriteForBody {
+    fn write(&mut self, d: &[u8]) -> io::Result<usize> {
+        let len = d.len();
+        println!("WriteForBody write len: {:?}", len);
+        if len <= 0 {
+            return Ok(len);
+        }
+        let d = Bytes::from(d.to_vec());
+        println!("WriteForBody write content: {:?}", d.clone());
+
+        {
+            let mut cached = self.cached.lock().unwrap();
+            cached.push_back(d);
+            cached.reserve_exact(10);
+        }
+        {
+            if let Some(waker) = self.waker.lock().unwrap().take() {
+                waker.wake();
+            }
+        }
+
+        /*
+        match self.0.try_send_data(d) {
+            Ok(_) => {
+                println!("WriteForBody write ok");
+            }
+            Err(_) => {
+                println!("WriteForBody write error");
+            }
+        }
+        */
+        Ok(len)
+    }
+
+    fn flush(&mut self) -> io::Result<()> {
+        {
+            if let Some(waker) = self.waker.lock().unwrap().take() {
+                waker.wake();
+            }
+        }
+
+        println!("flush ok");
+        Ok(())
+    }
+}
+
+#[cfg(feature = "multipart")]
+#[derive(Debug, Clone)]
+/// MultipartSerializerForStream Serialize the multipart body (for put/post/patch etc)
+pub struct MultipartSerializerForStream {
+    // NOTE: It can't be Copy because of this one:
+    thread_pool: Option<Arc<ThreadPool>>,
+}
+#[cfg(feature = "multipart")]
+impl BodySerializer<FormData, (String, Body)> for MultipartSerializerForStream
+where
+// B: HttpBody + Send + 'static,
+// B: From<Body>,
+// B::Data: Send,
+// B::Error: Into<Box<dyn StdError + Send + Sync>>,
+{
+    fn encode(&self, origin: FormData) -> StdResult<(String, Body), Box<dyn StdError>> {
+        // let mut data = Vec::<u8>::new();
+
+        let (tx, rx) = make_stream::<Bytes>();
+        let mut data = WriteForStream(tx);
+
+        /*
+        let (tx, body) = Body::channel();
+        let mut data = WriteForBody(Box::new(tx));
+        // */
+
+        /*
+        let mut data = WriteForBody {
+            cached: Arc::new(Mutex::new(VecDeque::with_capacity(10))),
+            waker: Arc::new(Mutex::new(None)),
+            alive: Arc::new(Mutex::new(AtomicBool::new(true))),
+        };
+        let body = data.clone();
+        */
+
+        let boundary = formdata::generate_boundary();
+        let boundary_thread = boundary.clone();
+        //*
+        // println!("Enter encode");
+        let _ = thread::spawn(move || {
+            // let _ = tokio::spawn(async move {
+            // let _ = tokio::spawn(async move {
+            // tokio::time::sleep(std::time::Duration::from_millis(200)).await;
+            // thread::sleep_ms(2000);
+            // data.0.send_data(Bytes::new()).await;
+
+            // println!("spawn: Some");
+            // println!("write_formdata begin");
+            match formdata::write_formdata(&mut data, &boundary_thread, &origin) {
+                Err(e) => println!("Error -> write_formdata {:?}", e),
+                _ => {}
+            };
+            // println!("write_formdata done");
+
+            match data.flush() {
+                Err(e) => println!("Error -> flush {:?}", e),
+                _ => {}
+            };
+            // println!("flush ok");
+
+            let mut tx = data.0;
+            tx.close_channel();
+            drop(tx);
+            // println!("Close!!");
+        });
+        // */
+        let content_type = get_content_type_from_multipart_boundary(boundary)?;
+
+        let body = rx
+            .map(|y| Ok::<Bytes, Box<dyn StdError + Send + Sync>>(y))
+            .into_stream();
+
+        // Ok((content_type, B::from(body)))
+        // Ok((content_type, body))
+        Ok((content_type, Body::wrap_stream(body)))
+    }
+}
+#[cfg(feature = "multipart")]
+#[allow(dead_code)]
+pub(crate) const DEFAULT_MULTIPART_SERIALIZER_FOR_STREAM: MultipartSerializerForStream =
+    MultipartSerializerForStream { thread_pool: None };
+
+pub struct HyperClient<C, B> {
+    pub client: Client<C, B>,
+    pub thread_pool: Option<ThreadPool>,
+}
 impl<C, B> BaseClient<Client<C, B>, Request<B>, Result<Response<Body>>, Method, HeaderMap, B>
     for HyperClient<C, B>
 where
@@ -52,10 +247,10 @@ where
     B::Error: Into<Box<dyn StdError + Send + Sync>>,
 {
     fn request(&self, req: Request<B>) -> Pin<Box<dyn Future<Output = Result<Response<Body>>>>> {
-        Box::pin(self.0.request(req))
+        Box::pin(self.client.request(req))
     }
     fn get_client(&mut self) -> &mut Client<C, B> {
-        return &mut self.0;
+        return &mut self.client;
     }
 }
 
@@ -110,9 +305,10 @@ impl
         Body,
     > {
         return SimpleHTTP::new_with_options(
-            Arc::new(Mutex::new(
-                HyperClient::<HttpConnector, Body>(Client::new()),
-            )),
+            Arc::new(Mutex::new(HyperClient::<HttpConnector, Body> {
+                client: Client::new(),
+                thread_pool: None,
+            })),
             VecDeque::new(),
             DEFAULT_TIMEOUT_MILLISECOND,
         );
@@ -202,15 +398,15 @@ impl Default
     }
 }
 
-#[derive(Debug)]
-pub struct HyperError(Error);
-impl StdError for HyperError {}
-
-impl std::fmt::Display for HyperError {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        self.0.fmt(f)
-    }
-}
+// #[derive(Debug)]
+// pub struct HyperError(Error);
+// impl StdError for HyperError {}
+//
+// impl std::fmt::Display for HyperError {
+//     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+//         self.0.fmt(f)
+//     }
+// }
 
 /*
 `CommonAPI` implements `make_api_response_only()`/`make_api_no_body()`/`make_api_has_body()`,
@@ -327,6 +523,7 @@ where
         .await
     }
 
+    /*
     pub async fn do_request_multipart(
         &self,
         method: Method,
@@ -360,6 +557,87 @@ where
         )
         .await
     }
+    // */
+}
+
+impl<C>
+    dyn BaseService<Client<C, Body>, Request<Body>, Result<Response<Body>>, Method, HeaderMap, Body>
+{
+    #[cfg(feature = "multipart")]
+    // NOTE: Experimental
+    #[allow(dead_code)]
+    pub(crate) fn make_api_multipart_for_stream<R>(
+        &self,
+        base: Arc<
+            dyn BaseService<
+                Client<C, Body>,
+                Request<Body>,
+                Result<Response<Body>>,
+                Method,
+                HeaderMap,
+                Body,
+            >,
+        >,
+        method: Method,
+        relative_url: impl Into<String>,
+        // request_serializer: Arc<dyn BodySerializer<FormData, (String, Body)>>,
+        response_deserializer: Arc<dyn BodyDeserializer<R>>,
+        _return_type: &R,
+    ) -> APIMultipart<
+        FormData,
+        R,
+        Client<C, Body>,
+        Request<Body>,
+        Result<Response<Body>>,
+        Method,
+        HeaderMap,
+        Body,
+    >
+    where
+        Body: From<Bytes>,
+    {
+        APIMultipart {
+            base,
+            method,
+            relative_url: relative_url.into(),
+            request_serializer: Arc::new(DEFAULT_MULTIPART_SERIALIZER_FOR_STREAM.clone()),
+            response_deserializer,
+        }
+    }
+
+    pub async fn do_request_multipart(
+        &self,
+        method: Method,
+        header: Option<HeaderMap>,
+        relative_url: impl Into<String>,
+        // content_type: impl Into<String>,
+        path_param: Option<impl Into<PathParam>>,
+        query_param: Option<impl Into<QueryParam>>,
+        body: FormData,
+    ) -> StdResult<Box<Body>, Box<dyn StdError>>
+    where
+        Body: From<Bytes>,
+    {
+        let (content_type, body) = DEFAULT_MULTIPART_SERIALIZER.encode(body)?;
+        self._call_common(
+            method,
+            header,
+            relative_url.into(),
+            content_type,
+            if let Some(v) = path_param {
+                Some(v.into())
+            } else {
+                None
+            },
+            if let Some(v) = query_param {
+                Some(v.into())
+            } else {
+                None
+            },
+            body,
+        )
+        .await
+    }
 }
 
 impl<C, B> CommonAPI<Client<C, B>, Request<B>, Result<Response<B>>, HeaderMap, B>
