@@ -11,18 +11,18 @@ use std::sync::{Arc, Mutex};
 use bytes::Bytes;
 use url::Url;
 
-#[cfg(feature = "multipart")]
-use formdata::FormData;
-
-#[cfg(feature = "for_serde")]
-use serde::{de::DeserializeOwned, Serialize};
-
 pub use super::common::{PathParam, QueryParam};
 use super::simple_http::{
     data_and_boundary_from_multipart, get_content_type_from_multipart_boundary, BaseClient,
     Interceptor, InterceptorFunc, SimpleHTTP,
 };
 
+#[cfg(feature = "multipart")]
+use formdata::FormData;
+
+#[cfg(feature = "for_serde")]
+use serde::{de::DeserializeOwned, Serialize};
+
 /*
 `BodySerializer  Serialize the body (for put/post/patch etc)
 */
@@ -44,13 +44,24 @@ impl BodySerializer<Bytes, Bytes> for DummyBypassSerializerForBytes {
         Ok(Bytes::from(origin))
     }
 }
-pub static DEFAULT_DUMMY_BYPASS_SERIALIZER_FOR_BYTES: DummyBypassSerializerForBytes =
+pub const DEFAULT_DUMMY_BYPASS_SERIALIZER_FOR_BYTES: DummyBypassSerializerForBytes =
     DummyBypassSerializerForBytes {};
 
 #[derive(Debug, Clone, Copy)]
-// DummyBypassSerializer Dummy bypass the body data, do nothing (for put/post/patch etc)
-pub struct DummyBypassSerializer {}
-impl<B> BodySerializer<Bytes, B> for DummyBypassSerializer
+// DummyBypassSerializerForBody Dummy bypass the body data, do nothing (for put/post/patch etc)
+pub struct DummyBypassSerializerForBody {}
+impl<B> BodySerializer<B, B> for DummyBypassSerializerForBody {
+    fn encode(&self, origin: B) -> StdResult<B, Box<dyn StdError>> {
+        Ok(origin)
+    }
+}
+pub const DEFAULT_DUMMY_BYPASS_SERIALIZER_FOR_BODY: DummyBypassSerializerForBody =
+    DummyBypassSerializerForBody {};
+
+#[derive(Debug, Clone, Copy)]
+// DummyBypassSerializerForBytesToBody Dummy bypass the body data, do nothing (for put/post/patch etc)
+pub struct DummyBypassSerializerForBytesToBody {}
+impl<B> BodySerializer<Bytes, B> for DummyBypassSerializerForBytesToBody
 where
     B: From<Bytes>,
 {
@@ -58,7 +69,8 @@ where
         Ok(B::from(origin))
     }
 }
-pub static DEFAULT_DUMMY_BYPASS_SERIALIZER: DummyBypassSerializer = DummyBypassSerializer {};
+pub const DEFAULT_DUMMY_BYPASS_SERIALIZER_FOR_BYTES_TO_BODY: DummyBypassSerializerForBytesToBody =
+    DummyBypassSerializerForBytesToBody {};
 
 #[derive(Debug, Clone, Copy)]
 /*
@@ -70,7 +82,7 @@ impl BodyDeserializer<Bytes> for DummyBypassDeserializer {
         Ok(Box::new(bytes.clone()))
     }
 }
-pub static DEFAULT_DUMMY_BYPASS_DESERIALIZER: DummyBypassDeserializer = DummyBypassDeserializer {};
+pub const DEFAULT_DUMMY_BYPASS_DESERIALIZER: DummyBypassDeserializer = DummyBypassDeserializer {};
 
 #[cfg(feature = "multipart")]
 #[derive(Debug, Clone, Copy)]
@@ -86,7 +98,7 @@ impl BodySerializer<FormData, (String, Bytes)> for MultipartSerializerForBytes {
     }
 }
 #[cfg(feature = "multipart")]
-pub static DEFAULT_MULTIPART_SERIALIZER_FOR_BYTES: MultipartSerializerForBytes =
+pub const DEFAULT_MULTIPART_SERIALIZER_FOR_BYTES: MultipartSerializerForBytes =
     MultipartSerializerForBytes {};
 
 #[cfg(feature = "multipart")]
@@ -105,7 +117,7 @@ where
     }
 }
 #[cfg(feature = "multipart")]
-pub static DEFAULT_MULTIPART_SERIALIZER: MultipartSerializer = MultipartSerializer {};
+pub const DEFAULT_MULTIPART_SERIALIZER: MultipartSerializer = MultipartSerializer {};
 
 #[cfg(feature = "for_serde")]
 #[derive(Debug, Clone, Copy)]
@@ -125,7 +137,7 @@ impl<T: Serialize> BodySerializer<T, Bytes> for SerdeJsonSerializerForBytes {
     }
 }
 #[cfg(feature = "for_serde")]
-pub static DEFAULT_SERDE_JSON_SERIALIZER_FOR_BYTES: SerdeJsonSerializerForBytes =
+pub const DEFAULT_SERDE_JSON_SERIALIZER_FOR_BYTES: SerdeJsonSerializerForBytes =
     SerdeJsonSerializerForBytes {};
 
 #[cfg(feature = "for_serde")]
@@ -140,7 +152,7 @@ where
     }
 }
 #[cfg(feature = "for_serde")]
-pub static DEFAULT_SERDE_JSON_SERIALIZER: SerdeJsonSerializer = SerdeJsonSerializer {};
+pub const DEFAULT_SERDE_JSON_SERIALIZER: SerdeJsonSerializer = SerdeJsonSerializer {};
 
 #[cfg(feature = "for_serde")]
 #[derive(Debug, Clone, Copy)]
@@ -155,7 +167,7 @@ impl<R: DeserializeOwned + 'static> BodyDeserializer<R> for SerdeJsonDeserialize
     }
 }
 #[cfg(feature = "for_serde")]
-pub static DEFAULT_SERDE_JSON_DESERIALIZER: SerdeJsonDeserializer = SerdeJsonDeserializer {};
+pub const DEFAULT_SERDE_JSON_DESERIALIZER: SerdeJsonDeserializer = SerdeJsonDeserializer {};
 
 pub trait BaseAPI<Client, Req, Res, Method, Header, B> {
     fn set_base_url(&mut self, url: Url);
@@ -551,7 +563,9 @@ impl<T, R, Client, Req, Res, Method, Header, B>
         Method: Clone,
     {
         // let mut sent_body = Box::new(sent_body);
+        println!("request_serializer encode before");
         let (content_type_with_boundary, sent_body) = self.request_serializer.encode(sent_body)?;
+        println!("request_serializer encode after");
         let body = self
             .base
             ._call_common(
@@ -573,6 +587,8 @@ impl<T, R, Client, Req, Res, Method, Header, B>
             )
             .await?;
 
+        println!("_call_common");
+
         // let mut target = Box::new(target);
         // let body = Box::new(body);
         // let bytes = hyper::body::to_bytes(*body).await?;
@@ -583,6 +599,8 @@ impl<T, R, Client, Req, Res, Method, Header, B>
         let bytes = result.ok().unwrap();
         let target = self.response_deserializer.decode(&bytes)?;
 
+        println!("response_deserializer.decode after");
+
         Ok(target)
     }
 }
